@@ -7,7 +7,8 @@ from pathlib import Path
 
 from pandas import to_datetime as _to_datetime
 
-from . import _FARSI_NORM, _MarketState, _csv2df, _F, _TypedDict, _get_data, \
+from . import _FARSI_NORM, _MarketState, _api, _csv2df, _F, _TypedDict, \
+    _get_data, \
     _numerize, _parse_market_state, _parse_ombud_messages, _rc, \
     _get, _StringIO, _BytesIO, _DF, _DataFrame, \
     _to_numeric, _read_html, _findall, _jstrptime, _get_par_tree, _jdatetime
@@ -285,8 +286,15 @@ class Instrument:
                 lineterminator=',')
         return result
 
-    async def trade_history(self, top: int) -> _DataFrame:
-        content = await _get_data(f'InstTradeHistory.aspx?i={self.code}&Top={top}')
+    async def trade_history(self, top: int, all_=False) -> _DataFrame:
+        """Get history of pmax, pmin, pc, pl, pf, py, tval, tvol, and tno.
+
+        :param top: number of top rows (days) to return
+        :param all_: include dates with no trade
+
+        Use ``Instrument.on_date(<date>).trades``.
+        """
+        content = await _get_data(f'InstTradeHistory.aspx?i={self.code}&Top={top}&A={all_:d}')
         df = _csv2df(
             _BytesIO(content)
             , sep='@'
@@ -308,6 +316,11 @@ class Instrument:
         """Get daily natural/legal history.
 
         In column names `n_` prefix stands for natural and `l_` for legal.
+
+        This method returns the information available at the "حقیقی-حقوقی" tab
+        of the instrument. It uses the `clienttype.aspx` module.
+
+        See also: ``Instrument.client_type_history``
         """
         return _csv2df(
             _BytesIO(await _get_data(f'clienttype.aspx?i={self.code}'))
@@ -318,6 +331,24 @@ class Instrument:
                 , 'n_buy_value', 'l_buy_value', 'n_sell_value', 'l_sell_value')
             , index_col='date', parse_dates=True , dtype='uint64')
 
+    async def client_type_history(self, date: int | str = None) -> _DataFrame | dict:
+        """Return natural/legal client type history.
+
+        :param date: Gregorian date in YYYYMMDD format. If None, return the
+        full history as a DataFrame. Otherwise, return the data for that
+        specific date as a dict.
+
+        Uses the information from api/ClientType/GetClientTypeHistory.
+
+        See also: ``Instrument.client_type``
+        """
+        if date is None:
+            j = await _api(f'ClientType/GetClientTypeHistory/{self.code}')
+            return _DataFrame(j['clientType'], copy=False)
+
+        j = await _api(f'ClientType/GetClientTypeHistory/{self.code}/{date}')
+        return j['clientType']
+
     async def identification(self) -> dict:
         """Return the information available in the identification (شناسه) tab.
 
@@ -343,9 +374,11 @@ class Instrument:
         return Instrument(ins_code, l18, l30)
 
     async def holders(self, cisin=None) -> _DataFrame:
-        """Get list of major unit/share holders.
+        """Get list of current major unit/shareholders.
 
         If `cisin` is not provided, it will be fetched using a web request.
+
+        See also: ``Instrument.on_date(<date>).holders``
         """
         if cisin is None:
             cisin = await self.cisin
@@ -411,6 +444,11 @@ class Instrument:
 
         For the meaning of instrument state codes refer to
             http://en.tsetmc.com/Site.aspx?ParTree=111411111Y
+
+        This method uses the old way of fetching intraday data from tsetmc.com
+        which is not used on the live site anymore.
+
+        For fetching individual parameters, use ``instrument.on_date(date)``.
         """
         text = await _get_par_tree(f'15131P&i={self.code}&d={date}')
         find = text.find
@@ -540,6 +578,97 @@ class Instrument:
             lambda col: [_jstrptime(i, format='%Y/%m/%d') for i in col])
         return df
 
+    def on_date(self, date: int | str) -> 'InstrumentOnDate':
+        """Return an object resembling Instrument on a specific date.
+
+        :param date: Gregorian date in YYYYMMDD format.
+        """
+        return InstrumentOnDate(self, date)
+
+
+class InstrumentOnDate:
+
+    __slots__ = 'date', 'code', 'inst'
+
+    def __init__(self, _inst: Instrument, _date: int | str):
+        """Return an object resembling Instrument on a specific date.
+
+        :param _inst: Instrument
+        :param _date: Gregorian date in YYYYMMDD format.
+
+        Users should not instantiate this class directly, but use
+        ``Instrument.on_date()`` instead.
+        """
+        self.date = _date
+        self.inst = _inst
+        self.code = _inst.code
+
+    async def closing_price(self) -> dict:
+        """Return general closing price info.
+
+        Result dict has the following keys: {
+        'priceChange', 'priceMin', 'priceMax', 'priceYesterday', 'priceFirst',
+        'last', 'id', 'insCode', 'dEven', 'hEven', 'pClosing', 'iClose',
+        'yClose', 'pDrCotVal', 'zTotTran', 'qTotTran5J', 'qTotCap'}
+        """
+        j = await _api(f'ClosingPrice/GetClosingPriceDaily/{self.code}/{self.date}')
+        return j['closingPriceDaily']
+
+    async def closing_price_history(self) -> _DataFrame:
+        """Get intraday closing price history."""
+        j = await _api(f'ClosingPrice/GetClosingPriceHistory/{self.code}/{self.date}')
+        return _DataFrame(j['closingPriceHistory'], copy=False)
+
+    async def states(self) -> _DataFrame:
+        """Get intraday instrument states.
+
+        http://www.tsetmc.com/Site.aspx?ParTree=111411111Y&LnkIdn=833
+        """
+        j = await _api(f'MarketData/GetInstrumentState/{self.code}/{self.date}')
+        return _DataFrame(j['instrumentState'], copy=False)
+
+    async def client_types(self) -> dict:
+        return await self.inst.client_type_history(self.date)
+
+    async def holders(self) -> _DataFrame:
+        """Return share/unit holders for a specific date and a day before that.
+
+        See also: ``Instrument.holders`` which returns the list of current
+        holders.
+        """
+        j = await _api(f'Shareholder/{self.code}/{self.date}')
+        return _DataFrame(j['shareShareholder'], copy=False)
+
+    async def best_limits(self) -> _DataFrame:
+        """Get intraday best limits history."""
+        j = await _api(f'BestLimits/{self.code}/{self.date}')
+        return _DataFrame(j['bestLimitsHistory'], copy=False)
+
+    async def trades(self) -> _DataFrame:
+        """Get intraday trades.
+
+        See also: ``Instrument.trade_history``
+        """
+        # todo: true vs false
+        j = await _api(f'Trade/GetTradeHistory/{self.code}/{self.date}/true')
+        return _DataFrame(j['tradeHistory'], copy=False)
+
+    async def static_thresholds(self) -> _DataFrame:
+        """Get intraday static thresholds."""
+        j = await _api(f'MarketData/GetStaticThreshold/{self.code}/{self.date}')
+        return _DataFrame(j['staticThreshold'], copy=False)
+
+    async def data(self) -> dict:
+        """Get general info about the instrument on the specific date.
+
+        The returned dict contains the following keys: {
+        'insCode', 'lVal30', 'lVal18AFC', 'flow', 'cIsin', 'zTitad', 'baseVol',
+        'instrumentID', 'cgrValCot', 'cComVal', 'lastDate', 'sourceID',
+        'flowTitle', 'cgrValCotTitle'}
+        """
+        j = await _api(f'Instrument/GetInstrumentHistory/{self.code}/{self.date}')
+        return j['instrumentHistory']
+
 
 async def price_adjustments(flow: int) -> _DataFrame:
     """Get price adjustments for a particular flow.
